#############################
# Disproportinality Functions

dif.G <- function(seat,vote.rate){
sqrt(sum((rate(seat)-vote.rate)^2)/2)
}

dif.LH <- function(seat,vote.rate){
sum(abs(rate(seat)-vote.rate))/2
}

dif.MN <- function(seat,vote.rate){
df <- length(seat)
pickup <- which(seat != 0)
seat <- seat[pickup]
vote.rate <- vote.rate[pickup]
2*sum(seat*(log(rate(seat))-log(vote.rate)))
}

dif.MN.std <- function(seat,vote.rate){
df <- length(seat)
pickup <- which(seat != 0)
seat <- seat[pickup]
vote.rate <- vote.rate[pickup]
log.like.ratio <- 2*sum(seat*(log(rate(seat))-log(vote.rate)))
pchisq(log.like.ratio,df)
}

dif.chi <- function(seat,vote.rate){
seat.prop <- vote.rate*sum(seat)
sum(((seat-seat.prop)^2)/seat.prop)
}

dif.chi.std <- function(seat,vote.rate){
df <- length(seat)
seat.prop <- vote.rate*sum(seat)
chisq <- sum(((seat-seat.prop)^2)/seat.prop)
pchisq(chisq,df)
}

dif.max <- function(seat,vote.rate){
 top <- which(vote.rate==max(vote.rate[-1]))
 mean((rate(seat)-vote.rate)[top])
}

dif.enp <- function(seat,vote.rate){
enp.vote(vote.rate)-enp.seat(seat)
}

enp.vote <-  function(vote.rate){
1/sum(vote.rate[-1]^2) # 𐔂Ȃ
}

enp.seat <-  function(seat){
1/(sum(rate(seat)[-1]^2)+(seat[1]/(sum(seat)^2))) # 𐔂Ȃ
}

# V~[Vp
dif.enp.sim <- function(seat,vote.rate){
enp.vote.sim(vote.rate)-enp.seat.sim(seat)
}

enp.vote.sim <-  function(vote.rate){
1/sum(vote.rate^2)
}

enp.seat.sim <-  function(seat){
1/(sum(rate(seat)^2))
}

dif.max.sim <- function(seat,vote.rate){
 top <- which(vote.rate==max(vote.rate))
 mean((rate(seat)-vote.rate)[top])
}

#############################
# Proportional Representaiton

forward <- function(vote,seat,seat.num,ballot){
  quota <- as.numeric(ifelse(((seat+1)>=ballot),(vote/(seat+1)),(vote/ballot))) 
  add <- which(quota == max(quota))
residual <- seat.num - (sum(seat)+length(add))
if(residual >= 0){seat[add] <- seat[add]+1}
if(residual < 0){seat[add] <- seat[add]+((seat.num - sum(seat))/(length(add)))}
  seat  
}

proportion <- function(vote,seat.num,ballot){
seat <- rep(0,length(vote))
  while (seat.num > sum(seat)){
    seat <- forward(vote,seat,seat.num,ballot)
  }
  seat
}

rate <- function(x){x/sum(x)}

##############
# Vote to Seat

vote.to.seat <- function(vote.mat,M,citizens,winner.vec){
  # ind version:
  ## seat.mat[5,] <- proportion(vote.rate.ind,sum(M))#prop.gerry.M 
  # vote.mat: matrix of v_ip
  # M: vector of district size
  # citizens: vector of numbers of eligible citizens
M.total <- sum(M)
obs <- nrow(vote.mat)
party.num <- ncol(vote.mat)
seat.dist <- seat.dist.prop <- seat.dist.gerry  <- seat.dist.prop.gerry  <- matrix(0,obs,party.num)
vote.rate <- rate(apply(vote.mat,2,sum))
vote.rate.ind <- c(0,rate(vote.rate[-1])) # independents not counted when votes are disperse
M.prop <- proportion(citizens,M.total,1) # proportional apportionment
# apportion seats at district level
for (i in 1:obs){
seat.dist[i,] <- proportion(vote.mat[i,],M[i],1)
seat.dist.prop[i,] <- proportion(vote.mat[i,],M.prop[i],1)
seat.dist.gerry[i,] <- proportion(vote.rate.ind,M[i],1) #independents are ignored
seat.dist.prop.gerry[i,] <- proportion(vote.rate.ind,M.prop[i],1) #independents are ignored
}
seat.mat <- matrix(NA,6,party.num)
seat.mat[1,] <- apply(seat.dist,2,sum)#base
seat.mat[2,] <- apply(seat.dist.prop,2,sum)#prop
seat.mat[3,] <- apply(seat.dist.gerry,2,sum)#gerry
seat.mat[4,] <- apply(seat.dist.prop.gerry,2,sum)#prop.gerry
seat.mat[5,] <- proportion(vote.rate,M.total,1)#prop.gerry.M 
seat.mat[6,] <- winner.vec #apply(seat.dist,2,sum)#real
rownames(seat.mat) <- c("base","prop","gerry","prop.gerry","prop.gerry.M","real")
# output
out <- list()
out$seat.mat <- seat.mat 
out$vote.rate <- vote.rate
out$M.prop <- M.prop
out$M <- M
out
}

# 1-6 block voting
#ballot <- M
vote.to.seat.block <- function(vote.mat,M,citizens,winner.vec){
M.total <- sum(M)
obs <- nrow(vote.mat)
party.num <- ncol(vote.mat)
seat.dist <- seat.dist.prop <- seat.dist.gerry  <- seat.dist.prop.gerry  <- matrix(0,obs,party.num)
vote.rate <- rate(apply(vote.mat,2,sum))
vote.rate.ind <- c(0,rate(vote.rate[-1])) # independents not counted when votes are disperse
M.prop <- proportion(citizens,M.total,1) # proportional apportionment
# apportion seats at district level
for (i in 1:obs){
seat.dist[i,] <- proportion(vote.mat[i,],M[i],M[i])
seat.dist.prop[i,] <- proportion(vote.mat[i,],M.prop[i],M.prop[i])
seat.dist.gerry[i,] <- proportion(vote.rate.ind,M[i],M[i]) #independents are ignored
seat.dist.prop.gerry[i,] <- proportion(vote.rate.ind,M.prop[i],M.prop[i]) #independents are ignored
}
# calculate national seats
seat.mat <- matrix(NA,6,party.num)
seat.mat[1,] <- apply(seat.dist,2,sum)#base
seat.mat[2,] <- apply(seat.dist.prop,2,sum)#prop
seat.mat[3,] <- apply(seat.dist.gerry,2,sum)#gerry
seat.mat[4,] <- apply(seat.dist.prop.gerry,2,sum)#prop.gerry
seat.mat[5,] <- proportion(vote.rate,M.total,1)#prop.gerry.M 
seat.mat[6,] <- winner.vec #apply(seat.dist,2,sum)#real
rownames(seat.mat) <- c("base","prop","gerry","prop.gerry","prop.gerry.M","real")
# output
out <- list()
out$seat.mat <- seat.mat 
out$vote.rate <- vote.rate
out$M.prop <- M.prop
out$M <- M
out
}

vote.to.seat.limit <- function(vote.mat,M,citizens,winner.vec,ballot){
M.total <- sum(M)
obs <- nrow(vote.mat)
party.num <- ncol(vote.mat)
seat.dist <- seat.dist.prop <- seat.dist.gerry  <- seat.dist.prop.gerry  <- matrix(0,obs,party.num)
vote.rate <- rate(apply(vote.mat,2,sum))
vote.rate.ind <- c(0,rate(vote.rate[-1])) # independents not counted when votes are disperse
M.prop <- proportion(citizens,M.total,1) # proportional apportionment
ballot.ave <- sum((apply(vote.mat,1,sum)/sum(vote.mat))*ballot)
# apportion seats at district level
for (i in 1:obs){
seat.dist[i,] <- proportion(vote.mat[i,],M[i],ballot[i])
seat.dist.prop[i,] <- proportion(vote.mat[i,],M.prop[i],ballot[i])
seat.dist.gerry[i,] <- proportion(vote.rate.ind,M[i],ballot[i]) #independents are ignored
seat.dist.prop.gerry[i,] <- proportion(vote.rate.ind,M.prop[i],ballot[i]) #independents are ignored
}
# calculate national seats
seat.mat <- matrix(NA,6,party.num)
seat.mat[1,] <- apply(seat.dist,2,sum)#base
seat.mat[2,] <- apply(seat.dist.prop,2,sum)#prop
seat.mat[3,] <- apply(seat.dist.gerry,2,sum)#gerry
seat.mat[4,] <- apply(seat.dist.prop.gerry,2,sum)#prop.gerry
seat.mat[5,] <- proportion(vote.rate,M.total,ballot.ave)#prop.gerry.M 
seat.mat[6,] <- winner.vec #apply(seat.dist,2,sum)#real
rownames(seat.mat) <- c("base","prop","gerry","prop.gerry","prop.gerry.M","real")
# output
out <- list()
out$seat.mat <- seat.mat 
out$vote.rate <- vote.rate
out$M.prop <- M.prop
out$M <- M
out
}

vote.to.seat.all <- function(data,winner,party,ballot){
output <- list()
data.num <- length(data)
for (kaiji in 1:data.num){
  data.sub <- data[[kaiji]]
M <- data.sub[,2]
citizens <- data.sub[,3]
vote.mat <- data.sub[,-c(1:3)]
winner.vec <- apply(winner[[kaiji]],2,sum)
output[[kaiji]] <- vote.to.seat.limit(vote.mat,M,citizens,winner.vec,ballot[[kaiji]])
colnames(output[[kaiji]]$seat.mat) <- party[[kaiji]]
}
output
}

###################
# Calculate Effects

eff.dif.all <- function(vote.to.seat.out){
  output <- list()
  kai.num <- length(vote.to.seat.out)
  eff.LH <- eff.MN <- eff.MN.std <- eff.chi <- eff.chi.std <- eff.enp <- eff.max <- eff.G <- matrix(NA,kai.num,6)
  for (kaiji in 1:kai.num){
output[[kaiji]] <- eff.dif(vote.to.seat.out[[kaiji]])
eff.LH[kaiji,] <- output[[kaiji]]$eff.LH
eff.MN[kaiji,] <- output[[kaiji]]$eff.MN
eff.MN.std[kaiji,] <- output[[kaiji]]$eff.MN.std
eff.chi[kaiji,] <- output[[kaiji]]$eff.chi
eff.chi.std[kaiji,] <- output[[kaiji]]$eff.chi.std
eff.enp[kaiji,] <- output[[kaiji]]$eff.enp
eff.max[kaiji,] <- output[[kaiji]]$eff.max
eff.G[kaiji,] <- output[[kaiji]]$eff.G
}
output$eff.LH <- eff.LH 
output$eff.MN <- eff.MN 
output$eff.MN.std <- eff.MN.std
output$eff.chi <- eff.chi
output$eff.chi.std <- eff.chi.std
output$eff.enp <- eff.enp
output$eff.max <- eff.max
output$eff.G <- eff.G
output
}

eff.dif <- function(vote.to.seat.out){
seat.mat <- vote.to.seat.out$seat.mat
vote.rate <- vote.to.seat.out$vote.rate
eff.LH <- seat.to.eff.dif(dif.LH,seat.mat,vote.rate)
eff.chi <- seat.to.eff.dif(dif.chi,seat.mat,vote.rate)
eff.chi.std <- seat.to.eff.dif(dif.chi.std,seat.mat,vote.rate)
eff.MN <- seat.to.eff.dif(dif.MN,seat.mat,vote.rate)
eff.MN.std <- seat.to.eff.dif(dif.MN.std,seat.mat,vote.rate)
eff.enp <- seat.to.eff.dif(dif.enp,seat.mat,vote.rate)
eff.max <- seat.to.eff.dif(dif.max,seat.mat,vote.rate)
eff.G <- seat.to.eff.dif(dif.G,seat.mat,vote.rate)
# output
out <- list()
out$seat.mat <- seat.mat 
out$eff.LH <- eff.LH 
out$eff.chi <- eff.chi
out$eff.chi.std <- eff.chi.std
out$eff.MN <- eff.MN
out$eff.MN.std <- eff.MN.std
out$eff.enp <- eff.enp
out$eff.max <- eff.max
out$eff.G <- eff.G
out$vote.rate <- vote.rate
out
}

seat.to.eff.dif <- function(dif.fun,seat.mat,vote.rate){
  dif.out <- eff <- rep(NA,6)
for (i in 1:6){
  dif.out[i] <- dif.fun(seat.mat[i,],vote.rate)
}  
eff[1] <- dif.out[3] - dif.out[4] #dif.gerry - dif.prop.gerry # malapportion
eff[2] <- dif.out[2] - dif.out[4] #dif.prop - dif.prop.gerry # gerrymander
eff[3] <- dif.out[1] - dif.out[4] - eff[1] - eff[2]#dif.base - dif.prop.gerry - eff[1] - eff[2] # interaction of malapportion and gerrymander
eff[4] <- dif.out[4] - dif.out[5] #dif.prop.gerry - dif.prop.gerry.M # districting
eff[5] <- dif.out[5] #dif.prop.gerry.M # representation
eff[6] <- dif.out[6] - dif.out[1] #dif.real - dif.base # strategic mistake
eff
}

##############
# Make Figures

make.fig.LH <- function(eff,main,ylab,where,year){
plot(year,eff[,4],type='l',ylim=c(min(eff),max(eff)),lty=2,lwd=2,xlab=c(""),ylab=ylab,main=main)#dis
polygon(c(year,rev(year)),c(apply(eff[,1:6],1,sum),rep(0,length(year))),col="gray")#total
lines(year,eff[,1],lty=1,lwd=2) #mal
lines(year,eff[,6],lty=1,lwd=4)#mistake
lines(year,eff[,3],lty=1,lwd=1) #inter
lines(year,eff[,2],lty=2,lwd=1)#ger
legend(where[1],where[2],c("s","萔","ϊ","","["),lwd=c(2,2,4,1,1),lty=c(2,1,1,1,2),bty=c("n"))
abline(v=(year.LH[6] +year.LH[7])/2, lty=1)
abline(v=(year.LH[13]+year.LH[14])/2,lty=1)
abline(v=(year.LH[15]+year.LH[16])/2,lty=1)
abline(v=(year.LH[21]+year.LH[22])/2,lty=1)
abline(v=(year.LH[22]+year.LH[23])/2,lty=1)
abline(v=(year.LH[40]+year.LH[41])/2,lty=1)
}

make.fig <- function(eff,main,ylab,where,year){
plot(year,eff[,4],type='l',ylim=c(min(eff),max(eff)),lty=2,lwd=2,xlab=c(""),ylab=ylab,main=main)#dis
polygon(c(year,rev(year)),c(apply(eff[,1:6],1,sum),rep(0,length(year))),col="gray")#total
lines(year,eff[,4],lty=2,lwd=2) #dis, again
lines(year,eff[,1],lty=1,lwd=2) #mal
lines(year,eff[,6],lty=1,lwd=4)#mistake
lines(year,eff[,3],lty=1,lwd=1) #inter
lines(year,eff[,2],lty=2,lwd=1)#ger
legend(where[1],where[2],c("s","萔","ϊ","","["),lwd=c(2,2,4,1,1),lty=c(2,1,1,1,2),bty=c("n"))
}

make.jpeg.LH <- function(eff,main,ylab,where,year,title){
jpeg(title, width = 480, height = 480,
         pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
plot(year,eff[,4],type='l',ylim=c(min(eff),max(eff)),lty=2,lwd=2,xlab=c(""),ylab=ylab,main=main)#dis
polygon(c(year,rev(year)),c(apply(eff[,1:6],1,sum),rep(0,length(year))),col="gray")#total
lines(year,eff[,1],lty=1,lwd=2) #mal
lines(year,eff[,6],lty=1,lwd=4)#mistake
lines(year,eff[,3],lty=1,lwd=1) #inter
lines(year,eff[,2],lty=2,lwd=1)#ger
legend(where[1],where[2],c("s","萔","ϊ","","["),lwd=c(2,2,4,1,1),lty=c(2,1,1,1,2),bty=c("n"))
abline(v=(year.LH[6] +year.LH[7])/2, lty=1)
abline(v=(year.LH[13]+year.LH[14])/2,lty=1)
abline(v=(year.LH[15]+year.LH[16])/2,lty=1)
abline(v=(year.LH[21]+year.LH[22])/2,lty=1)
abline(v=(year.LH[22]+year.LH[23])/2,lty=1)
abline(v=(year.LH[40]+year.LH[41])/2,lty=1)
dev.off()
}

make.jpeg <- function(eff,main,ylab,where,year,title){
jpeg(title, width = 480, height = 480,
         pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
plot(year,eff[,4],type='l',ylim=c(min(eff),max(eff)),lty=2,lwd=2,xlab=c(""),ylab=ylab,main=main)#dis
polygon(c(year,rev(year)),c(apply(eff[,1:6],1,sum),rep(0,length(year))),col="gray")#total
lines(year,eff[,1],lty=1,lwd=2) #mal
lines(year,eff[,6],lty=1,lwd=4)#mistake
lines(year,eff[,3],lty=1,lwd=1) #inter
lines(year,eff[,2],lty=2,lwd=1)#ger
legend(where[1],where[2],c("s","萔","ϊ","","["),lwd=c(2,2,4,1,1),lty=c(2,1,1,1,2),bty=c("n"))
dev.off()
}

make.jpeg.PR <- function(eff,main,ylab,where,year,title){
eff <- eff[,-6]
jpeg(title, width = 480, height = 480,
         pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
plot(year,eff[,4],type='l',ylim=c(min(eff),max(eff)),lty=2,lwd=2,xlab=c(""),ylab=ylab,main=main)#dis
polygon(c(year,rev(year)),c(apply(eff[,1:5],1,sum),rep(0,length(year))),col="gray")#total
lines(year,eff[,4],lty=2,lwd=2) #dis, again
lines(year,eff[,1],lty=1,lwd=2) #mal
lines(year,eff[,3],lty=1,lwd=1) #inter
lines(year,eff[,2],lty=2,lwd=1)#ger
legend(where[1],where[2],c("s","萔","","["),lwd=c(2,2,1,1),lty=c(2,1,1,2),bty=c("n"))
dev.off()
}

IV.figure <- function(var,where,main,ylab){
IV.out.var <- c(IV.out.LH[,var])
ylim <- c(min(IV.out.var),max(IV.out.var))
plot(year.LH,IV.out.LH[,var],type='l',lwd=4,ylim=ylim,xlab=c(""),main=main,ylab=ylab)
abline(v=(year.LH[6]+year.LH[7])/2,lty=1)
abline(v=(year.LH[13]+year.LH[14])/2,lty=1)
abline(v=(year.LH[15]+year.LH[16])/2,lty=1)
abline(v=(year.LH[21]+year.LH[22])/2,lty=1)
abline(v=(year.LH[22]+year.LH[23])/2,lty=1)
abline(v=(year.LH[40]+year.LH[41])/2,lty=1)
}

IV.figure.jpeg <- function(var,where,main,ylab,title){
jpeg(title, width = 480, height = 480,
         pointsize = 12, bg = "white", res = NA, restoreConsole = TRUE)
IV.figure(var,where,main,ylab)
dev.off()
}

sign.cor <- function(x,y){
cor(sign(x-mean(x)),sign(y-mean(y)))
}
